/*
SoundPlayer.cpp

Copyright (C) 2004 Vince Tagle <vtagle@newsguy.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of 
the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
SOFTWARE.
*/

#include "SoundPlayer.h"
#include <OpenAL/al.h>
#include <OpenAL/alut.h>
#include "OSXDefaultFileLocation.h"

tSoundPlayer gSoundPlayer; 

// Since we're only doing simple sounds in this game, there's no need to do
// any of the positional stuff
// 
ALfloat SourcePos[] = { 0.0, 0.0, 0.0 };
ALfloat SourceVel[] = { 0.0, 0.0, 0.0 };
ALfloat ListenerPos[] = { 0.0, 0.0, 0.0 };
ALfloat ListenerVel[] = { 0.0, 0.0, 0.0 };
ALfloat ListenerOri[] = { 0.0, 0.0, -1.0,  0.0, 1.0, 0.0 };

/*! \brief  Container for an audio buffer and its source for easy manipulation using OpenAL. 
*/
class tSoundData
{
public:
	ALuint  fBuffer;
	ALuint  fSource; 
	
	tSoundData(ALuint buffer, ALuint source) : fBuffer(buffer), fSource(source)
	{
	}
	
	~tSoundData()
	{
		alDeleteBuffers(1, &fBuffer);
		alDeleteSources(1, &fSource);
	}
};

tSoundPlayer::tSoundPlayer()
{
}

tSoundPlayer::~tSoundPlayer()
{
	tSoundDataIter soundIter; 
	for (soundIter = fSounds.begin(); soundIter != fSounds.end(); ++soundIter)
	{
		delete (*soundIter).second; 
	}
	
	fSounds.clear(); 
	alutExit();
}

void tSoundPlayer::mInit(int* argc, char** argv)
{
	SetPathToResourcesFolder();
	
	// Initialize the OpenAL component
	// 
	alutInit(argc, argv); 
	
	// tell OpenAl where the listener is going to be
	// 
	alListenerfv(AL_POSITION,    ListenerPos);
	alListenerfv(AL_VELOCITY,    ListenerVel);
	alListenerfv(AL_ORIENTATION, ListenerOri);
	
	alListenerf(AL_GAIN, 0.75f); 
}

void tSoundPlayer::mPlaySound(SInt32 soundToken, bool loop)
{
	if ( fSounds.find(soundToken) != fSounds.end() )
	{
		alSourcei(fSounds[soundToken]->fSource, AL_LOOPING, loop);
		alSourcePlay(fSounds[soundToken]->fSource); 
	}
}

void tSoundPlayer::mStopSound(SInt32 soundToken)
{
	if ( fSounds.find(soundToken) != fSounds.end() )
	{
		alSourceStop(fSounds[soundToken]->fSource); 
	}
}

bool tSoundPlayer::mIsPlaying(SInt32 soundToken)
{
	bool returnValue = false; 
	
	if ( fSounds.find(soundToken) != fSounds.end() )
	{
		ALint result;
		alGetSourcei(fSounds[soundToken]->fSource, AL_SOURCE_STATE, &result); 
		returnValue = result == AL_PLAYING ? true : false; 
	}
	
	return returnValue; 
}

void tSoundPlayer::mPreloadSound(SInt32 soundToken, char* filename)
{
	tSoundData* soundData = mCreateSoundDataObject(filename); 
	if ( soundData )
	{
		fSounds.insert( tSoundDataMap::value_type(soundToken, soundData) ); 
	}
}

void tSoundPlayer::mSetSoundPosition(SInt32 soundToken, float x, float y, float z)
{
	if ( fSounds.find(soundToken) != fSounds.end() )
	{
		alSource3f(fSounds[soundToken]->fSource, AL_POSITION, x, y, z);
	}
}

void tSoundPlayer::mGetSoundPosition(SInt32 soundToken, float &x, float &y, float &z)
{
	if ( fSounds.find(soundToken) != fSounds.end() )
	{
		alGetSource3f(fSounds[soundToken]->fSource, AL_POSITION, &x, &y, &z);
	}
}

void tSoundPlayer::mSetListenerPosition(float x, float y, float z)
{
	alListener3f(AL_POSITION, x, y, z);
}

void tSoundPlayer::mGetListenerPosition(float &x, float &y, float &z)
{
	alGetListener3f(AL_POSITION, &x, &y, &z);
}

tSoundData* tSoundPlayer::mCreateSoundDataObject(char* filename)
{
	// Buffers to hold sound data.
	// 
	ALuint Buffer;
	
	// Sources are points of emitting sound.
	// 
	ALuint Source;
	
	ALenum format;
	ALsizei size;
	ALvoid* data;
	ALsizei freq;
	
	// Load wav data into a buffer.
	// 
	alGenBuffers(1, &Buffer);

	if (alGetError() != AL_NO_ERROR )
	{
		return NULL;
	}
	
	alutLoadWAVFile(filename, &format, &data, &size, &freq);
	alBufferData(Buffer, format, data, size, freq);
	alutUnloadWAV(format, data, size, freq);
	
	// Bind the buffer with the source.
	// 
	alGenSources(1, &Source);
	
	if (alGetError() != AL_NO_ERROR)
	{
		return NULL;
	}
	
	alSourcei (Source, AL_BUFFER,   Buffer   );
	alSourcef (Source, AL_PITCH,    1.0      );
	alSourcef (Source, AL_GAIN,     1.0      );
	alSourcefv(Source, AL_POSITION, SourcePos);
	alSourcefv(Source, AL_VELOCITY, SourceVel);
	alSourcei (Source, AL_LOOPING,  false    );
	
	// Do another error check and return.
	//
	if (alGetError() == AL_NO_ERROR)
	{
		return new tSoundData(Buffer, Source); 
	}
	else
	{
		return NULL; 
	}
}
